@@ -22,23 +22,21 @@ RIGHT_WING_TAIL = ")"
 class BinaryTreeIterator:
     Mode = Enum("Mode", ["DFS", "BFS", "PRE_ORDER", "IN_ORDER", "POST_ORDER"])
 
-    def __init__(self, root=None,
-        mode: Mode = Mode.DFS,
-        is_left_first: bool = True,
-        data_name: str = DATA_NAME,
-        left_name: str = LEFT_NAME,
-        right_name: str = RIGHT_NAME
+    def __init__(self, node=None, mode: Mode = Mode.DFS, is_left_first: bool = True,
+        data_name: str = DATA_NAME, left_name: str = LEFT_NAME, right_name: str = RIGHT_NAME
     ):
-        self.root = root
         self.data_name = data_name
         self.left_name = left_name
         self.right_name = right_name
+        self.child_selector = self._left_first if is_left_first else self._right_first
         if mode == BinaryTreeIterator.Mode.DFS or mode == BinaryTreeIterator.Mode.PRE_ORDER:
-            self.generator = self._pre_order_left_first(root) if is_left_first else self._pre_order_right_first(root)
+            self.generator = self._pre_order(node)
         elif mode == BinaryTreeIterator.Mode.IN_ORDER:
-            self.generator = self._in_order_left_first(root) if is_left_first else self._in_order_right_first(root)
+            self.generator = self._in_order(node)
         elif mode == BinaryTreeIterator.Mode.POST_ORDER:
-            self.generator = self._post_order_left_first(root) if is_left_first else self._post_order_right_first(root)
+            self.generator = self._post_order(node)
+        else:  # BFS
+            self.generator = self._bfs(node)
 
     def __iter__(self):
         return self
@@ -46,41 +44,51 @@ class BinaryTreeIterator:
     def __next__(self):
         return next(self.generator).__dict__[self.data_name]
 
-    def _pre_order_left_first(self, node):
+    def _left_first(self, node):
+        if node is not None:
+            yield node.__dict__[self.left_name]
+            yield node.__dict__[self.right_name]
+
+    def _right_first(self, node):
+        if node is not None:
+            yield node.__dict__[self.right_name]
+            yield node.__dict__[self.left_name]
+
+    def _pre_order(self, node):
         if node is not None:
             yield node
-            yield from self._pre_order_left_first(node.__dict__[self.left_name])
-            yield from self._pre_order_left_first(node.__dict__[self.right_name])
+            for child in self.child_selector(node):
+                yield from self._pre_order(child)
 
-    def _pre_order_right_first(self, node):
+    def _in_order(self, node):
         if node is not None:
-            yield node
-            yield from self._pre_order_right_first(node.__dict__[self.right_name])
-            yield from self._pre_order_right_first(node.__dict__[self.left_name])
+            can_yield = True
+            for child in self.child_selector(node):
+                yield from self._in_order(child)
+                if can_yield:
+                    yield node
+                can_yield = False
 
-    def _in_order_left_first(self, node):
+    def _post_order(self, node):
         if node is not None:
-            yield from self._in_order_left_first(node.__dict__[self.left_name])
-            yield node
-            yield from self._in_order_left_first(node.__dict__[self.right_name])
-
-    def _in_order_right_first(self, node):
-        if node is not None:
-            yield from self._in_order_right_first(node.__dict__[self.right_name])
-            yield node
-            yield from self._in_order_right_first(node.__dict__[self.left_name])
-
-    def _post_order_left_first(self, node):
-        if node is not None:
-            yield from self._post_order_left_first(node.__dict__[self.left_name])
-            yield from self._post_order_left_first(node.__dict__[self.right_name])
+            for child in self.child_selector(node):
+                yield from self._post_order(child)
             yield node
 
-    def _post_order_right_first(self, node):
+    # def _bfs(self, node, depth):
+    #     if node is not None and depth < self.height:
+    #         yield node
+    #         for child in self._bfs(node, depth + 1):
+    #             yield from self.child_selector(child)
+    def _bfs(self, node):
         if node is not None:
-            yield from self._post_order_right_first(node.__dict__[self.right_name])
-            yield from self._post_order_right_first(node.__dict__[self.left_name])
-            yield node
+            queue = deque([node])
+            while len(queue) > 0:
+                node = queue.popleft()
+                yield node
+                for child in self.child_selector(node):
+                    if child is not None:
+                        queue.append(child)
 
 
 class BinaryTree(object):
@@ -92,7 +100,7 @@ class BinaryTree(object):
     def __init__(self, root=None,
         data_name: str = DATA_NAME,
         left_name: str = LEFT_NAME,
-        right_name: str = LEFT_NAME,
+        right_name: str = RIGHT_NAME,
         iterator_mode: BinaryTreeIterator.Mode = BinaryTreeIterator.Mode.DFS,
         iterator_is_left_first: bool = True,
         algorithm: BinaryTreeAlgorithm = None
@@ -107,17 +115,17 @@ class BinaryTree(object):
 
     def __iter__(self):
         return BinaryTreeIterator(
-            root=self.root,
+            node=self.root,
             mode=self.iterator_mode,
             is_left_first=self.iterator_is_left_first,
             data_name=self.data_name,
             left_name=self.left_name,
             right_name=self.right_name
-        )
+        )  # New iterator every time instead of keeping an iterator instance since self.root might change overtime
 
     def __reversed__(self):
         return BinaryTreeIterator(
-            root=self.root,
+            node=self.root,
             mode=self.iterator_mode,
             is_left_first=(not self.iterator_is_left_first),
             data_name=self.data_name,
@@ -125,19 +133,18 @@ class BinaryTree(object):
             right_name=self.right_name
         )
 
-    def clear(self):
-        self.root = None
+    def __contains__(self, data) -> bool:
+        return any(d == data for d in iter(self))
+
+    def __str__(self) -> str:
+        return self.to_string()
 
     def new_node(self, data, left=None, right=None):
         node = self.BinaryTreeNode()
-        node.__dict__ = {
-            self.data_name: data,
-            self.left_name: left,
-            self.right_name: right
-        }
+        node.__dict__ = {self.data_name: data, self.left_name: left, self.right_name: right}
         return node
 
-    def node_to_string(self, node):
+    def node_to_string(self, node) -> str:
         return str(self.get_data(node))
 
     def get_data(self, node):
@@ -158,22 +165,83 @@ class BinaryTree(object):
     def set_right(self, node, right):
         node.__dict__[self.right_name] = right
 
+    def get_left_most(self, node):
+        """ O(logN) """
+        if node is None:
+            raise ValueError(f"Invalid node: {node}")
+        while self.get_left(node) is not None:
+            node = self.get_left(node)
+        return node
+
+    def get_right_most(self, node):
+        """ O(logN) """
+        if node is None:
+            raise ValueError(f"Invalid node: {node}")
+        while self.get_right(node) is not None:
+            node = self.get_right(node)
+        return node
+
+    def get_depth(self, node) -> int:
+        """
+            count nodes on the path from root to the given node
+            this method should be overridden if the node has a parent pointer
+        """
+        pass
+
+    def get_height(self, node) -> int:
+        """ count nodes on the path from the given node to its furthest leave """
+        if node is None:
+            return 0
+        return max(self.get_height(self.get_left(node)), self.get_height(self.get_right(node))) + 1
+
     def print(self,
-        left_wing: str = LEFT_WING, right_wing: str = RIGHT_WING,
-        left_wing_head: str = LEFT_WING_HEAD, right_wing_head: str = RIGHT_WING_HEAD,
-        left_wing_tail: str = LEFT_WING_TAIL, right_wing_tail: str = RIGHT_WING_TAIL,
+        left_wing: str = LEFT_WING,
+        left_wing_head: str = LEFT_WING_HEAD,
+        left_wing_tail: str = LEFT_WING_TAIL,
+        right_wing: str = RIGHT_WING,
+        right_wing_head: str = RIGHT_WING_HEAD,
+        right_wing_tail: str = RIGHT_WING_TAIL,
         node_to_string: Callable = None
     ):
-        BinaryTreePrinter(
-            data_name=self.data_name, left_name=self.left_name, right_name=self.right_name,
-            left_wing=left_wing, right_wing=right_wing,
-            left_wing_head=left_wing_head, right_wing_head=right_wing_head,
-            left_wing_tail=left_wing_tail, right_wing_tail=right_wing_tail,
-            node_to_string=self.node_to_string if node_to_string is None else node_to_string
-        ).print(self.root)
+        print(
+            "\n" + self.to_string(
+                left_wing=left_wing,
+                left_wing_head=left_wing_head,
+                left_wing_tail=left_wing_tail,
+                right_wing=right_wing,
+                right_wing_head=right_wing_head,
+                right_wing_tail=right_wing_tail,
+                node_to_string=node_to_string
+            )
+        )
 
-    def depth(self):
-        return self.algorithm.depth(self.root)
+    def to_string(self,
+        left_wing: str = LEFT_WING,
+        left_wing_head: str = LEFT_WING_HEAD,
+        left_wing_tail: str = LEFT_WING_TAIL,
+        right_wing: str = RIGHT_WING,
+        right_wing_head: str = RIGHT_WING_HEAD,
+        right_wing_tail: str = RIGHT_WING_TAIL,
+        node_to_string: Callable = None
+    ):
+        return BinaryTreePrinter(
+            data_name=self.data_name,
+            left_name=self.left_name,
+            right_name=self.right_name,
+            left_wing=left_wing,
+            left_wing_head=left_wing_head,
+            left_wing_tail=left_wing_tail,
+            right_wing=right_wing,
+            right_wing_head=right_wing_head,
+            right_wing_tail=right_wing_tail,
+            node_to_string=self.node_to_string if node_to_string is None else node_to_string
+        ).to_string(self.root)
+
+    def clear(self):
+        self.root = None
+
+    def height(self):
+        return self.get_height(self.root)
 
     def is_balanced(self) -> bool:
         return self.algorithm.is_balanced(self.root)[0]
@@ -280,68 +348,6 @@ class BinaryTree(object):
                 other_queue.append(self.get_right(other_node))
         return BinaryTree(other_root, self.data_name, self.left_name, self.right_name)
 
-    def remove_bst_node(self, data):
-        # self.root = self.algorithm.remove_bst_node(self.root, data)
-        parent, node = None, self.root
-        while node is not None:
-            if data < self.get_data(node):
-                parent, node = node, self.get_left(node)
-            elif data > self.get_data(node):
-                parent, node = node, self.get_right(node)
-            else:
-                if self.get_right(node) is None:
-                    left_child = self.get_left(node)
-                    if parent is None:
-                        self.root = left_child
-                    elif node == self.get_left(parent):
-                        self.set_left(parent, left_child)
-                    else:  # node == self.get_right(parent)
-                        self.set_right(parent, left_child)
-                else:  # left most node in the right subtree
-                    left_most_parent, left_most = node, self.get_right(node)
-                    while left_most is not None and self.get_left(left_most) is not None:
-                        left_most_parent = left_most
-                        left_most = self.get_left(left_most)
-                    self.set_data(node, self.get_data(left_most))  # swap data then delete left_most
-                    right_child = self.get_right(left_most)  # left_most only have the right child
-                    if left_most == self.get_right(node):
-                        self.set_right(node, right=right_child)  # left_most_parent == node
-                    else:
-                        self.set_left(left_most_parent, left=right_child)
-                break
-
-    def remove_bst_nodes(self, data_lower_bound, data_upper_bound):
-        if data_upper_bound < data_lower_bound:
-            raise ValueError(f"data_upper_bound {data_upper_bound} is smaller than data_lower_bound {data_lower_bound}")
-        parent, node = None, self.root
-        while node is not None:
-            data = self.get_data(node)
-            if data < data_lower_bound:
-                parent, node = node, self.get_right(node)
-            elif data_upper_bound < data:
-                parent, node = node, self.get_left(node)
-            else:
-                if self.get_right(node) is None:
-                    left_child = self.get_left(node)
-                    if parent is None:
-                        self.root = left_child
-                    elif node == self.get_left(parent):
-                        self.set_left(parent, left_child)
-                    else:  # node == self.get_right(parent)
-                        self.set_right(parent, left_child)
-                    node = left_child
-                else:  # left_most only have the right child
-                    left_most_parent, left_most = node, self.get_right(node)
-                    while left_most is not None and self.get_left(left_most) is not None:
-                        left_most_parent = left_most
-                        left_most = self.get_left(left_most)
-                    self.set_data(node, self.get_data(left_most))  # swap data then delete left_most
-                    right_child = self.get_right(left_most)  # left_most only have the right child
-                    if left_most == self.get_right(node):
-                        self.set_right(node, right=right_child)  # left_most_parent == node
-                    else:
-                        self.set_left(left_most_parent, left=right_child)
-
 
 class BinaryTreeAlgorithm:
     """ Recursion Helpers """
@@ -391,11 +397,6 @@ class BinaryTreeAlgorithm:
             return level
         return 0
 
-    def depth(self, root):
-        if root is None:
-            return 0
-        return max(self.depth(root.__dict__[self.left_name]), self.depth(root.__dict__[self.right_name])) + 1
-
     def subtree_sum_extremum(self, root, extremum_func):
         if root is None:
             return 0, 0
@@ -476,20 +477,25 @@ class BinaryTreePrinter:
             self.node = node
 
     def __init__(self,
-        data_name: str = DATA_NAME, left_name: str = LEFT_NAME, right_name: str = RIGHT_NAME,
-        left_wing: str = LEFT_WING, right_wing: str = RIGHT_WING,
-        left_wing_head: str = LEFT_WING_HEAD, right_wing_head: str = RIGHT_WING_HEAD,
-        left_wing_tail: str = LEFT_WING_TAIL, right_wing_tail: str = RIGHT_WING_TAIL,
+        data_name: str = DATA_NAME,
+        left_name: str = LEFT_NAME,
+        right_name: str = RIGHT_NAME,
+        left_wing: str = LEFT_WING,
+        left_wing_head: str = LEFT_WING_HEAD,
+        left_wing_tail: str = LEFT_WING_TAIL,
+        right_wing: str = RIGHT_WING,
+        right_wing_head: str = RIGHT_WING_HEAD,
+        right_wing_tail: str = RIGHT_WING_TAIL,
         node_to_string: Callable = None
     ):
         self.data_name = data_name
         self.left_name = left_name
         self.right_name = right_name
         self.left_wing = left_wing
-        self.right_wing = right_wing
         self.left_wing_head = left_wing_head
-        self.right_wing_head = right_wing_head
         self.left_wing_tail = left_wing_tail
+        self.right_wing = right_wing
+        self.right_wing_head = right_wing_head
         self.right_wing_tail = right_wing_tail
         self.tree_depth = 0
         self.max_data_string_length = 0
